# vim:set fileencoding=utf-8 ft=python ts=8 sw=4 sts=4 et cindent:

# fsmi.base -- Wrap the main fsmi database (as needed).
#
# Copyright © 2010  Philipp Kern <pkern@debian.org>
# Copyright © 2010  Fabian Knittel <fabian.knittel@avona.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

from __future__ import absolute_import

from garfield.database import reflect_tables
import sqlalchemy
import sqlalchemy.sql as sql
import sqlalchemy.orm as orm
from sqlalchemy.schema import Column, ForeignKey
from sqlalchemy.types import Integer, String


def setup_tables(_sess, meta):
    tables = reflect_tables(meta, ['benutzer', ('acl', 'rechte')])

    # Manual definitions for views from which sqlalchemy cannot infer the pkey.
    tables['effektive_benutzer_rechte'] = sqlalchemy.Table(
            'effektive_benutzer_rechte',
            meta,
            Column('rechteid', Integer, ForeignKey('acl.rechte.rechteid'),
                primary_key=True),
            Column('benutzer_id', Integer, ForeignKey('benutzer.benutzer_id'),
                primary_key=True),
            schema='acl')
    tables['benutzer_primaer_email_adresse'] = sqlalchemy.Table(
            'benutzer_primaer_email_adresse',
            meta,
            Column('benutzer_id', Integer, ForeignKey('benutzer.benutzer_id'),
                primary_key=True),
            Column('email_adresse', String))

    orm.mapper(Recht, tables['rechte'])
    orm.mapper(EffektivesBenutzerRecht, tables['effektive_benutzer_rechte'])
    orm.mapper(BenutzerPrimaerEmailAdresse,
            tables['benutzer_primaer_email_adresse'])
    orm.mapper(Benutzer, tables['benutzer'],
            properties={
                'primaer_email_adresse': orm.relation(
                    BenutzerPrimaerEmailAdresse)
            })

class Benutzer(object):
    """@DynamicAttrs"""
    @staticmethod
    def associate_user_account_with_garfield(sess, benutzer_id,
            garfield_user_id):
        fun = sql.func.associate_user_account_with_garfield
        return sess.execute(fun(benutzer_id, garfield_user_id))

    def primary_email_address(self):
        return self.primaer_email_adresse[0].email_adresse

    @classmethod
    def get_by_garfield_user_id(cls, sess, garfield_user_id):
        return sess.query(cls).filter(cls.garfield_user_id == garfield_user_id)\
                .first()


class Recht(object):
    """@DynamicAttrs"""
    pass

class EffektivesBenutzerRecht(object):
    """@DynamicAttrs"""
    pass

class BenutzerPrimaerEmailAdresse(object):
    """@DynamicAttrs"""
    pass
